/*
 * Copyright (c) 2005 - 2007, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.tranql.connector.oracle;

import java.sql.SQLException;

import oracle.jdbc.pool.OracleDataSource;
import oracle.jdbc.xa.client.OracleXADataSource;
import org.tranql.connector.jdbc.AbstractXADataSourceMCF;

/**
 * ManagedConnectionFactory for connecting to a Oracle database with XA transactions.
 *
 * @version $Revision: 663 $ $Date: 2009-09-24 23:48:55 -0700 (Thu, 24 Sep 2009) $
 */
public class XAMCF extends AbstractXADataSourceMCF<OracleXADataSource> {
    private String password;

    /**
     * Default constructor for an Oracle XA DataSource.
     *
     * @throws SQLException if there was a problem instantiating an OracleXADataSource
     */
    public XAMCF() throws SQLException {
        super(new OracleXADataSource(), new OracleExceptionSorter());
    }

    public void setDatabaseName(String dbname) {
        xaDataSource.setDatabaseName(dbname);
    }

    public String getDatabaseName() {
        return xaDataSource.getDatabaseName();
    }

    public void setServiceName(String serviceName) {
        xaDataSource.setServiceName(serviceName);
    }

    public String getServiceName() {
        return xaDataSource.getServiceName();
    }

    public void setDataSourceName(String dsname) {
        xaDataSource.setDataSourceName(dsname);
    }

    public String getDataSourceName() {
        return xaDataSource.getDataSourceName();
    }

    public void setDescription(String desc) {
        xaDataSource.setDescription(desc);
    }

    public String getDescription() {
        return xaDataSource.getDescription();
    }

    public void setNetworkProtocol(String np) {
        xaDataSource.setNetworkProtocol(np);
    }

    public String getNetworkProtocol() {
        return xaDataSource.getNetworkProtocol();
    }

    public void setPassword(String pwd) {
        this.password = pwd;
        xaDataSource.setPassword(pwd);
    }

    public String getPassword() {
        return password;
    }

    public void setPortNumber(Integer pn) {
        xaDataSource.setPortNumber(pn == null ? 0 : pn.intValue());
    }

    public Integer getPortNumber() {
        return new Integer(xaDataSource.getPortNumber());
    }

    public void setServerName(String sn) {
        xaDataSource.setServerName(sn);
    }

    public String getServerName() {
        return xaDataSource.getServerName();
    }

    public void setUserName(String user) {
        xaDataSource.setUser(user);
    }

    public String getUserName() {
        return xaDataSource.getUser();
    }

    public String getDriverType() {
        return xaDataSource.getDriverType();
    }

    public void setDriverType(String s) {
        xaDataSource.setDriverType(s);
    }

    public String getTNSEntryName() {
        return xaDataSource.getTNSEntryName();
    }

    public void setTNSEntryName(String s) {
        xaDataSource.setTNSEntryName(s);
    }

    public String getURL() {
        try {
            return xaDataSource.getURL();
        } catch (SQLException e) {
            return "<unknown location>";
        }
    }

    public Integer getMaxStatements() throws SQLException {
        return new Integer(xaDataSource.getMaxStatements());
    }

    public void setMaxStatements(Integer ms) throws SQLException {
        int i = ms == null ? 0 : ms.intValue();
        xaDataSource.setMaxStatements(i);
        xaDataSource.setImplicitCachingEnabled(i != 0);
    }

    /**
     * Override the assumption that xa implementations are spec compliant since apparently Oracle xa is not
     * @return
     */
    public Boolean isCommitBeforeAutocommit() {
        return Boolean.TRUE;
    }


    /**
     * Equality is defined in terms of the serverName, portNumber and databaseName properties
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof XAMCF) {
            OracleDataSource other = ((XAMCF) obj).xaDataSource;
            return equals(xaDataSource.getServerName(), other.getServerName()) &&
                    xaDataSource.getPortNumber() == other.getPortNumber() &&
                    equals(xaDataSource.getDatabaseName(), other.getDatabaseName());
        } else {
            return false;
        }
    }

    private static boolean equals(String a, String b) {
        return a == b || a != null && a.equals(b);
    }

    public int hashCode() {
        return hashCode(xaDataSource.getServerName()) ^ xaDataSource.getPortNumber() ^ hashCode(xaDataSource.getDatabaseName());
    }

    private static int hashCode(String s) {
        return s == null ? 0 : s.hashCode();
    }

    public String toString() {
        return "XAMCF[" + getURL() + "]";
    }
}
